use std::path::Path;

use crate::Author;

pub struct Mailmap {
	entries: Vec<MapEntry>,
}

#[derive(Debug)]
struct MapEntry {
	new_name: Option<String>,
	new_email: Option<String>,
	old_name: Option<String>,
	old_email: String,
}

impl Mailmap {
	pub fn new(repo_path: &Path) -> Self {
		let mailmap_path = repo_path.join(".mailmap");
		let mailmap = std::fs::read_to_string(mailmap_path).unwrap_or_default();
		let mailmap = gix::mailmap::parse(mailmap.as_bytes());

		let mut entries: Vec<MapEntry> = vec![];
		for entry in mailmap {
			let Ok(entry) = entry
			else {
				continue;
			};

			let entry = MapEntry {
				new_name: entry.new_name().map(|v| v.to_string()),
				new_email: entry.new_email().map(|v| v.to_string()),
				old_name: entry.old_name().map(|v| v.to_string()),
				old_email: entry.old_email().to_string(),
			};

			entries.push(entry);
		}

		Self { entries }
	}

	pub fn resolve(&self, author: Author) -> Author {
		for entry in &self.entries {
			if let Some(old_name) = &entry.old_name
				&& old_name == &author.name
			{
				return Author {
					name: entry.new_name.as_ref().unwrap_or(&author.name).to_string(),
					email: entry
						.new_email
						.as_ref()
						.unwrap_or(&author.email)
						.to_string(),
				};
			}
		}

		author
	}
}
